searchNodes=[{"doc":"Defines a macro to easily define a Kafka Broadway pipeline in your application, configuring Broadway and Kafka via callbacks. It starts a Broadway pipeline for each topic and partition for increased concurrency processing events, receiving partition assignments from the group coordinator and starting an Elsa group supervisor for each. It uses the following callbacks: kafka_config/1 receives start_link options and returns the Kafka consumer configuration which is passed to Elsa.Supervisor.start_link/1 . broadway_config/3 receives a keyword list of configuration options, a topic and a partition. It returns the keyword list to configure the Broadway processors, batchers and contexts. Called by OffBroadway.Kafka.ShowtimeHandler . For example: defmodule ShowtimeBroadway do use OffBroadway.Kafka def kafka_config ( _opts ) do [ connection : :per_partition , endpoints : [ localhost : 9092 ] , group_consumer : [ group : &quot;per_partition&quot; , topics : [ &quot;topic1&quot; ] , config : [ prefetch_count : 5 , prefetch_bytes : 0 , begin_offset : :earliest ] ] ] end def broadway_config ( opts , topic , partition ) do [ name : :&quot;broadway_per_partition_ \#{ topic } _ \#{ partition } &quot; , processors : [ default : [ concurrency : 5 ] ] , context : %{ pid : Keyword . get ( opts , :pid ) } ] end def handle_message ( processor , message , context ) do send ( context . pid , { :message , message } ) message end end","ref":"OffBroadway.Kafka.html","title":"OffBroadway.Kafka","type":"behaviour"},{"doc":"Macro which starts pipeline for each Elsa consumer group manager instantiated.","ref":"OffBroadway.Kafka.html#__using__/1","title":"OffBroadway.Kafka.__using__/1","type":"macro"},{"doc":"","ref":"OffBroadway.Kafka.html#c:broadway_config/3","title":"OffBroadway.Kafka.broadway_config/3","type":"callback"},{"doc":"","ref":"OffBroadway.Kafka.html#c:kafka_config/1","title":"OffBroadway.Kafka.kafka_config/1","type":"callback"},{"doc":"Implements the Broadway acknowledger behaviour, acking messages back to Kafka once they have been successfully processed. It stores message ack references in ETS as an ordered set, acknowledging them in the order received rather than the order processed. This ensures that a failure of the Broadway pipeline does not allow messages received later but processed faster to erroneously mark lost messages as acknowledged when they should instead be reprocessed on recovery of the pipeline.","ref":"OffBroadway.Kafka.Acknowledger.html","title":"OffBroadway.Kafka.Acknowledger","type":"module"},{"doc":"Acknowledges processed messages to Kafka. Due to Kafka's requirement to maintain message order for proper offset management, concatenates successful and failed messages together and stores the total offset to return for acknowledgement.","ref":"OffBroadway.Kafka.Acknowledger.html#ack/3","title":"OffBroadway.Kafka.Acknowledger.ack/3","type":"function"},{"doc":"Constructs an ack_ref record for storing the status of message acknowledgement in ETS.","ref":"OffBroadway.Kafka.Acknowledger.html#ack_ref/1","title":"OffBroadway.Kafka.Acknowledger.ack_ref/1","type":"function"},{"doc":"Adds a set of messages, represented by a contiguous range, to ETS for tracking acknowledgement in the proper order.","ref":"OffBroadway.Kafka.Acknowledger.html#add_offsets/2","title":"OffBroadway.Kafka.Acknowledger.add_offsets/2","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"OffBroadway.Kafka.Acknowledger.html#child_spec/1","title":"OffBroadway.Kafka.Acknowledger.child_spec/1","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.Acknowledger.html#is_empty?/1","title":"OffBroadway.Kafka.Acknowledger.is_empty?/1","type":"function"},{"doc":"Creates an acknowledger GenServer process and links it to the current process.","ref":"OffBroadway.Kafka.Acknowledger.html#start_link/1","title":"OffBroadway.Kafka.Acknowledger.start_link/1","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.Acknowledger.html#t:ack_ref/0","title":"OffBroadway.Kafka.Acknowledger.ack_ref/0","type":"type"},{"doc":"Defines an Elsa.Consumer.MessageHandler for integrating with Broadway in the traditional way, using Broadway directly. It supplies a simple pass-through init function and delegates message handling to the OffBroadway.Kafka.Producer module. Set up by OffBroadway.Kafka.Producer .","ref":"OffBroadway.Kafka.ClassicHandler.html","title":"OffBroadway.Kafka.ClassicHandler","type":"module"},{"doc":"","ref":"OffBroadway.Kafka.ClassicHandler.html#connection/0","title":"OffBroadway.Kafka.ClassicHandler.connection/0","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ClassicHandler.html#generation_id/0","title":"OffBroadway.Kafka.ClassicHandler.generation_id/0","type":"function"},{"doc":"Handles Kafka messages. Delegates message processing to OffBroadway.Kafka.Producer.handle_messages/2 .","ref":"OffBroadway.Kafka.ClassicHandler.html#handle_messages/2","title":"OffBroadway.Kafka.ClassicHandler.handle_messages/2","type":"function"},{"doc":"Supplies a basic init function, delegating the rest to the default Elsa.Consumer.MessageHandler behavior.","ref":"OffBroadway.Kafka.ClassicHandler.html#init/1","title":"OffBroadway.Kafka.ClassicHandler.init/1","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ClassicHandler.html#partition/0","title":"OffBroadway.Kafka.ClassicHandler.partition/0","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ClassicHandler.html#topic/0","title":"OffBroadway.Kafka.ClassicHandler.topic/0","type":"function"},{"doc":"Defines a Broadway Producer module which receives messages from Kafka to initiate the Broadway pipeline. Sends messages to the handle_info/2 and handle_demand/2 functions based on requests and tracks acknowledgements in state.","ref":"OffBroadway.Kafka.Producer.html","title":"OffBroadway.Kafka.Producer","type":"module"},{"doc":"Passes Kafka partition assignment changes to the handle_info/2 function. Called by Elsa assignments_revoked_handler .","ref":"OffBroadway.Kafka.Producer.html#assignments_revoked/1","title":"OffBroadway.Kafka.Producer.assignments_revoked/1","type":"function"},{"doc":"Handles message events based on demand. Updates the demand based on the existing demand in state and sends the requested number of message events.","ref":"OffBroadway.Kafka.Producer.html#handle_demand/2","title":"OffBroadway.Kafka.Producer.handle_demand/2","type":"function"},{"doc":"Handles assignments revoked by the Kafka broker. Waits until acknowledgers report as empty and cleans out any events in queue.","ref":"OffBroadway.Kafka.Producer.html#handle_info/2","title":"OffBroadway.Kafka.Producer.handle_info/2","type":"function"},{"doc":"Passes Kafka messages to the handle_info/2 function. Called by Elsa message handler.","ref":"OffBroadway.Kafka.Producer.html#handle_messages/2","title":"OffBroadway.Kafka.Producer.handle_messages/2","type":"function"},{"doc":"Names the producer process and initializes state for the producer GenServer. If args contains a value for :endpoints , creates a handler config and passes it to the Elsa library to start a consumer group supervisor and stores the returned pid in the Broadway Producer state for reference.","ref":"OffBroadway.Kafka.Producer.html#init/1","title":"OffBroadway.Kafka.Producer.init/1","type":"function"},{"doc":"Starts an OffBroadway.Kafka producer process linked to the current process.","ref":"OffBroadway.Kafka.Producer.html#start_link/1","title":"OffBroadway.Kafka.Producer.start_link/1","type":"function"},{"doc":"Handles termination of the Elsa consumer group supervisor process if one exists.","ref":"OffBroadway.Kafka.Producer.html#terminate/2","title":"OffBroadway.Kafka.Producer.terminate/2","type":"function"},{"doc":"Defines an Elsa.Consumer.MessageHandler for integrating with Broadway which supports creating a pipeline per topic partition for increased concurrency. Configuration for Kafka and Broadway are handled by the OffBroadway.Kafka.kafka_config/1 and OffBroadway.Kafka.broadway_config/3 callbacks. Set up by the OffBroadway.Kafka macro.","ref":"OffBroadway.Kafka.ShowtimeHandler.html","title":"OffBroadway.Kafka.ShowtimeHandler","type":"module"},{"doc":"","ref":"OffBroadway.Kafka.ShowtimeHandler.html#connection/0","title":"OffBroadway.Kafka.ShowtimeHandler.connection/0","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ShowtimeHandler.html#generation_id/0","title":"OffBroadway.Kafka.ShowtimeHandler.generation_id/0","type":"function"},{"doc":"Handles Kafka messages. Delegates message processing to OffBroadway.Kafka.Producer.handle_messages/2 .","ref":"OffBroadway.Kafka.ShowtimeHandler.html#handle_messages/2","title":"OffBroadway.Kafka.ShowtimeHandler.handle_messages/2","type":"function"},{"doc":"Initializes Broadway pipeline for Elsa.Consumer.MessageHandler . Assumes a single producer stage using the OffBroadway.Kafka.Producer module. Other Broadway configuration comes from the OffBroadway.Kafka.broadway_config/3 callback. Retains a reference to the producer process in the state.","ref":"OffBroadway.Kafka.ShowtimeHandler.html#init/1","title":"OffBroadway.Kafka.ShowtimeHandler.init/1","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ShowtimeHandler.html#partition/0","title":"OffBroadway.Kafka.ShowtimeHandler.partition/0","type":"function"},{"doc":"","ref":"OffBroadway.Kafka.ShowtimeHandler.html#topic/0","title":"OffBroadway.Kafka.ShowtimeHandler.topic/0","type":"function"},{"doc":"This library integrates the Broadway data processing pipeline library with Kafka . It communicates with Kafka using the Elsa Elixir library, which itself uses the Brod Erlang library. It can dynamically create Broadway concurrency stages on a per-topic or per-partition basis for a given Kafka topic.","ref":"readme.html","title":"OffBroadwayKafka","type":"extras"},{"doc":"Add off_broadway_kafka to the list of dependencies in mix.exs : def deps do [ { :off_broadway_kafka , &quot;~&gt; 2.0&quot; } ] end Docs can be found on HexDocs or generated with ExDoc .","ref":"readme.html#installation","title":"OffBroadwayKafka - Installation","type":"extras"},{"doc":"ClassicHandler This example uses Broadway in the traditional way. It calls Elsa.Supervisor.start_link/1 with the specified kafka_config to set up a Kafka consumer. It receives Kafka messages and adds them as events to the OffBroadway.Kafka.Producer Broadway Producer, which are then handled by the specified Broadway pipeline. defmodule ClassicBroadway do use Broadway def start_link ( opts ) do kafka_config = [ connection : :client1 , endpoints : [ localhost : 9092 ] , group_consumer : [ group : &quot;classic&quot; , topics : [ &quot;topic1&quot; ] , config : [ begin_offset : :earliest ] ] ] Broadway . start_link ( __MODULE__ , name : __MODULE__ , producer : [ module : { OffBroadway.Kafka.Producer , kafka_config } , concurrency : 1 ] , processors : [ default : [ concurrency : 1 ] ] , context : %{ pid : Keyword . get ( opts , :pid ) } ) end def handle_message ( processor , message , context ) do send ( context . pid , { :message , message } ) message end end ShowtimeHandler This example uses the OffBroadway.Kafka macro. It starts a Broadway pipeline for each topic and partition for increased concurrency processing events. defmodule ShowtimeBroadway do use OffBroadway.Kafka def kafka_config ( _opts ) do [ connection : :per_partition , endpoints : [ localhost : 9092 ] , group_consumer : [ group : &quot;per_partition&quot; , topics : [ &quot;topic1&quot; ] , config : [ prefetch_count : 5 , prefetch_bytes : 0 , begin_offset : :earliest ] ] ] end def broadway_config ( opts , topic , partition ) do [ name : :&quot;broadway_per_partition_ \#{ topic } _ \#{ partition } &quot; , processors : [ default : [ concurrency : 5 ] ] , context : %{ pid : Keyword . get ( opts , :pid ) } ] end def handle_message ( processor , message , context ) do send ( context . pid , { :message , message } ) message end end","ref":"readme.html#examples","title":"OffBroadwayKafka - Examples","type":"extras"}]